@@ -3,7 +3,7 @@ Soome basic utilities.
 """
 
 
-def saving_method(exp) -> bool:
+def saving_method(exp) -> str:
     if not exp.secrets.getboolean("mongo_saving_agent", "use"):
         if exp.config.getboolean("local_saving_agent", "use"):
             return "local"
@@ -20,8 +20,10 @@ class MatchingTimeout(Exception):
 class MatchingError(Exception):
     pass
 
+
 class BusyGroup(Exception):
     pass
 
+
 class NoMatch(Exception):
-    pass
\ No newline at end of file
+    pass
