@@ -4,23 +4,19 @@ Functionality related to groups.
 import json
 import random
 import time
-from traceback import format_exception
-from itertools import chain
 from collections import UserDict
-from uuid import uuid4
-from pathlib import Path
-from typing import Iterator, List
 from dataclasses import asdict, dataclass, field
-from enum import Enum, auto
-from abc import ABC, abstractproperty
+from itertools import chain
+from pathlib import Path
+from traceback import format_exception
+from typing import Iterator, List
+from uuid import uuid4
 
 from pymongo.collection import ReturnDocument
 
-from .member import MemberManager
-from .member import GroupMember
-from ._util import MatchingError, BusyGroup
-from ._util import saving_method
+from ._util import BusyGroup, MatchingError, saving_method
 from .element import Chat
+from .member import GroupMember, MemberManager
 
 
 class GroupType:
@@ -115,13 +111,12 @@ class SharedGroupData(UserDict):
 
         else:
             msg = (
-                f"You are setting or getting item '{key}', which refers "
-                f"to an object of type {type(value)}. This is most likely a mutable type. "
-                "Using a mutable object in SharedGroupData is very dangerous. "
-                "You may end up losing data, because SharedGroupData can only write "
-                "to the database when an item is set, not when it is changed. "
-                "Please switch to using only immutable types, like "
-                "tuple, str, int, float, and bool."
+                f"You are setting or getting item '{key}', which refers to an object of"
+                f" type {type(value)}. This is most likely a mutable type. Using a"
+                " mutable object in SharedGroupData is very dangerous. You may end up"
+                " losing data, because SharedGroupData can only write to the database"
+                " when an item is set, not when it is changed. Please switch to using"
+                " only immutable types, like tuple, str, int, float, and bool."
             )
             self.group.exp.log.warning(msg)
 
@@ -220,7 +215,7 @@ class GroupIO(GroupHelper):
         return self.db.find_one(self.query, {"_id": False})
 
     def _load_local(self) -> dict:
-        with open(self.path, "r", encoding="utf-8") as fp:
+        with open(self.path, encoding="utf-8") as fp:
             return json.load(fp)
 
     def load_markbusy(self) -> GroupData:
@@ -298,13 +293,14 @@ class GroupRoles(GroupHelper):
     def open(self) -> Iterator[str]:
         finished = self.manager.find_finished_sessions(self.data.members)
         pending = self.manager.find_active_sessions(self.data.members)
-        sessions = chain(finished, pending)
+        sessions = list(chain(finished, pending))
         roles = (role for role, sid in self.roles.items() if sid not in sessions)
         return roles
 
     def assign(self, role: str, member: GroupMember):
-        self.data.roles[role] = member.data.session_id
+        self.group.data.roles[role] = member.data.session_id
         member.data.role = role
+        self.group.io.save()
 
     def next(self) -> str:
         role = next(self.open(), None)
@@ -314,6 +310,7 @@ class GroupRoles(GroupHelper):
         role_list = list(self.data.roles.items())
         random.shuffle(role_list)
         self.data.roles = dict(role_list)
+        self.group.io.save()
 
 
 class GroupMemberManager(GroupHelper):
@@ -345,12 +342,21 @@ class GroupMemberManager(GroupHelper):
         """
         if len(self.data.roles) > 2:
             raise MatchingError(
-                "Can't use 'you' for groups with more than 2 slots, because it is ambiguous. Use the 'other_members()' generator instead, or access members based on their roles."
+                "Can't use 'you' for groups with more than 2 slots, because it is"
+                " ambiguous. Use the 'other_members()' generator instead, or access"
+                " members based on their roles."
             )
-        else:
-            you = next(self.active_other_members(), None)
-            if not you:
-                you = next(self.other_members(), None)
+
+        you = next(self.finished_other_members(), None)
+        if you:
+            return you
+
+        you = next(self.active_other_members(), None)
+        if you:
+            return you
+
+        you = next(self.other_members(), None)
+        if you:
             return you
 
     @property
@@ -372,23 +378,16 @@ class GroupMemberManager(GroupHelper):
         GroupMember: Returns the :class:`.GroupMember` that inhabits the
         given role.
         """
-        if not role in self.data.roles:
+        if role not in self.data.roles:
             raise AttributeError(f"Role '{role}' not found in {self}.")
 
-        active = (m for m in self.active_members() if m.data.role == role)
-        member = next(active, None)
-
-        if not member:
-            inactive = (m for m in self.members() if m.data.role == role)
-            member = next(inactive, None)
-
-        return member
+        return next(self.manager.find([self.group.data.roles[role]]), None)
 
     def members(self) -> Iterator[GroupMember]:
         return self.manager.find(self.data.members)
 
     def active_members(self) -> Iterator[GroupMember]:
-        sessions = self.manager.find_active_sessions(self.data.members)
+        sessions = list(self.manager.find_active_sessions(self.data.members))
         return self.manager.find(sessions)
 
     def other_members(self) -> Iterator[GroupMember]:
@@ -397,7 +396,17 @@ class GroupMemberManager(GroupHelper):
                 yield member
 
     def active_other_members(self) -> Iterator[GroupMember]:
-        sessions = self.manager.find_active_sessions(self.data.members)
+        sessions = list(self.manager.find_active_sessions(self.data.members))
+        if not sessions:
+            return
+        for member in self.manager.find(sessions):
+            if not member.data.session_id == self.exp.session_id:
+                yield member
+
+    def finished_other_members(self) -> Iterator[GroupMember]:
+        sessions = list(self.manager.find_finished_sessions(self.data.members))
+        if not sessions:
+            return
         for member in self.manager.find(sessions):
             if not member.data.session_id == self.exp.session_id:
                 yield member
@@ -426,26 +435,26 @@ class Group:
         See :class:`.GroupMember` for information about member objects.
 
     Examples:
-        
+
         In this basic example, we access both group members through their
         roles and print their inputs on the last page::
 
             import alfred3 as al
             import alfred3_interact as ali
 
-            exp = Experiment()
+            exp = al.Experiment()
 
             @exp.setup
             def setup(exp):
                 spec = ali.SequentialSpec("a", "b", nslots=10, name="spec1")
                 exp.plugins.mm = ali.MatchMaker(spec, exp=exp)
-            
+
             @exp.member
             class Demo:
 
                 def on_exp_access(self):
                     self += al.TextEntry(leftlab="Enter some text", force_input=True, name="el1")
-            
+
             @exp.member
             class Match(ali.WaitingPage):
 
@@ -453,14 +462,14 @@ class Group:
                     group = self.exp.plugins.mm.match()
                     self.exp.plugins.group = group
                     return True
-            
+
             @exp.member
             class Success(al.Page):
 
                 def on_first_show(self):
                     group = self.exp.plugins.group
                     role = group.me.role
-                    
+
                     self += al.Text(f"Successfully matched to role: {role}")
 
                     a = group.a
@@ -468,8 +477,9 @@ class Group:
 
                     self += al.Text(f"Values of group member a: {a.values}")
                     self += al.Text(f"Values of group member b: {b.values}")
-                    
+
     """
+
     def __init__(self, matchmaker, **data):
         self.mm = matchmaker
         self.exp = self.mm.exp
@@ -499,7 +509,7 @@ class Group:
         data["exp_id"] = exp_id if exp_id is not None else self.exp.exp_id
         data["matchmaker_id"] = mm_id if mm_id is not None else self.mm.matchmaker_id
 
-        if not "exp_version" in data:
+        if "exp_version" not in data:
             data["exp_version"] = self.mm.exp_version
 
         data.pop("_id", None)
@@ -633,11 +643,11 @@ class Group:
 
         Yields:
             :class:`.GroupMember`
-        
+
         See Also:
             - :meth:`.active_members`
             - :meth:`.other_members`
-        
+
         """
         return self.groupmember_manager.members()
 
@@ -660,7 +670,7 @@ class Group:
         except for :attr:`.me`. Yields :class:`.GroupMember` objects.
 
         Yields:
-            :class:`.GroupMember`        
+            :class:`.GroupMember`
         """
         return self.groupmember_manager.other_members()
 
@@ -671,7 +681,7 @@ class Group:
 
         Yields:
             :class:`.GroupMember`
-        
+
         See Also:
             - :attr:`.GroupMember.active`
         """
@@ -682,7 +692,10 @@ class Group:
         nmembers = self.groupmember_manager.nactive if self.data.members else 0
         shortid = self.data.group_id[-4:]
 
-        return f"{type(self).__name__}(roles={roles}, active members={nmembers}, id='{shortid}')"
+        return (
+            f"{type(self).__name__}(roles={roles}, active members={nmembers},"
+            f" id='{shortid}')"
+        )
 
     def __getattr__(self, name):
         return self.groupmember_manager.get_member_by_role(role=name)
@@ -691,7 +704,9 @@ class Group:
         return self.groupmember_manager.get_member_by_role(role=key)
 
     def __eq__(self, other):
-        return (type(self) == type(other)) and (self.data.group_id == other.data.group_id)
+        return (type(self) == type(other)) and (
+            self.data.group_id == other.data.group_id
+        )
 
     def __iadd__(self, member):
         # TODO: Add group ID to member's experiment data
@@ -704,6 +719,11 @@ class Group:
         self.io.save()
         return self
 
+    def deactivate(self):
+        with self as group:
+            group.data.active = False
+            group.io.save()
+
     def __enter__(self):
         data = self.io.load_markbusy()
         i = 0
@@ -725,10 +745,8 @@ class Group:
         if exc_type:
             tb = "".join(format_exception(exc_type, exc_value, tb))
             self.exp.log.error(
-                (
-                    f"There was an error when operating on {self}: {exc_value}."
-                    "The group was deactivated.\n{tb}"
-                )
+                f"There was an error when operating on {self}: {exc_value}."
+                "The group was deactivated.\n{tb}"
             )
             self.data.active = False
             self.io.save()
@@ -783,12 +801,11 @@ class GroupManager:
                 continue
 
             else:
-                with open(fpath, "r", encoding="utf-8") as f:
+                with open(fpath, encoding="utf-8") as f:
                     d = json.load(f)
-                    if (
-                        d["exp_version"] == self.mm.exp_version
-                        and d["spec_name"] == self.spec_name
-                    ):
+                    version_matches = d["exp_version"] == self.mm.exp_version
+                    spec_matches = d["spec_name"] == self.spec_name
+                    if version_matches and spec_matches:
                         yield d
 
     def _mongo_groups(self) -> Iterator[dict]:
