@@ -2,18 +2,16 @@
 Functionality related to group members.
 """
 
-import time
 import datetime
 import json
-from typing import Iterator, List
+import time
 from dataclasses import asdict, dataclass, field
+from typing import Iterator, List
 
-from pymongo.collection import ReturnDocument
 from alfred3.data_manager import DataManager as dm
 from alfred3.quota import SessionGroup
 
 from ._util import saving_method
-from ._util import MatchingError
 
 
 @dataclass
@@ -61,7 +59,7 @@ class GroupMemberIO(MemberHelper):
         self.member.data = GroupMemberData(**data)
 
     def _load_local(self) -> dict:
-        with open(self.path, "r", encoding="utf-8") as f:
+        with open(self.path, encoding="utf-8") as f:
             return json.load(f)
 
     def _load_mongo(self) -> dict:
@@ -75,7 +73,7 @@ class GroupMemberIO(MemberHelper):
             self._save_local()
 
     def _save_local(self):
-        with open(self.path, "r", encoding="utf-8") as f:
+        with open(self.path, encoding="utf-8") as f:
             mm = json.load(f)
 
         sid = self.member.data.session_id
@@ -96,6 +94,7 @@ class GroupMemberIO(MemberHelper):
         self.db.find_one_and_update(self.query, [{"$set": data}])
         self.member.data.ping = now
 
+
 # TODO Manuell deaktivieren für MatchMaker-Chaining
 @dataclass
 class GroupMemberStatus:
@@ -141,7 +140,6 @@ class GroupMemberStatus:
 
 
 class GroupMemberExpData(MemberHelper):
-    
     @property
     def db(self):
         return self.exp.db_main
@@ -174,7 +172,7 @@ class GroupMemberExpData(MemberHelper):
             if isinstance(projection, list):
                 data = {k: v for k, v in data.items() if k in projection}
             elif isinstance(projection, dict):
-                data = {k: v for k, v in data.items() if projection[k]}
+                data = {k: v for k, v in data.items() if projection.get(k, True)}
 
         return data
 
@@ -227,7 +225,7 @@ class GroupMemberInfo(MemberHelper):
             return time_print
 
         return None
-    
+
     @property
     def last_save(self) -> float:
         projection = {"exp_save_time": True, "_id": False}
@@ -245,7 +243,7 @@ class GroupMemberInfo(MemberHelper):
 class GroupMember:
     """
     The group member object grants access to a member's experiment data.
-    
+
     The group member object's most important job is to provide easy
     access to the member's experiment data through the following
     attributes. They provide the same objects as the corresponding
@@ -258,6 +256,7 @@ class GroupMember:
     - :attr:`.client_data`
     - :attr:`.adata`
     """
+
     def __init__(self, matchmaker, **data):
 
         self.mm = matchmaker
@@ -292,7 +291,7 @@ class GroupMember:
         bool: Indicates whether the member is associated with a group.
         """
         return self.status.matched
-    
+
     @property
     def group_id(self) -> str:
         """
@@ -307,12 +306,12 @@ class GroupMember:
         dict: Flat dictionary of input element values.
 
         Gives access to the member's experiment inputs.
-        
+
         See Also:
-            The dict works just like 
+            The dict works just like
             :attr:`alfred3.experiment.ExperimentSession.values`. The keys are
             the names of input elements in the member's experiment session.
-            The values are the user inputs. 
+            The values are the user inputs.
         """
         projection = {}
         projection.update({key: False for key in dm._client_data_keys})
@@ -320,14 +319,14 @@ class GroupMember:
 
         data = self.expdata.load(projection)
         return dm.flatten(data)
-    
+
     @property
     def session_data(self) -> dict:
         """
         dict: Full dictionary of experiment session data.
 
         See Also:
-            The dict works just like 
+            The dict works just like
             :attr:`alfred3.experiment.ExperimentSession.session_data`.
         """
         return self.expdata.load()
@@ -338,7 +337,7 @@ class GroupMember:
         dict: Dictionary of client data.
 
         See Also:
-            The dict works just like 
+            The dict works just like
             :attr:`alfred3.experiment.ExperimentSession.client_data`.
         """
         client_data = list(dm._client_data_keys)
@@ -374,7 +373,7 @@ class GroupMember:
         dict: Dictionary of additional data.
 
         See Also:
-            The dict works just like 
+            The dict works just like
             :attr:`alfred3.experiment.ExperimentSession.adata`.
         """
         data = self.expdata.load(["additional_data"])
@@ -434,52 +433,62 @@ class MemberManager:
         q = self.query_exp
         q["exp_finished"] = False
         q["exp_aborted"] = False
-        earliest_start = time.time() - self.exp.session_timeout
-        q["$or"] = [{"exp_start_time": {"$gte": earliest_start}}, {"exp_start_time": None}]
 
         if sessions is not None:
             q["exp_session_id"] = {"$in": sessions}
 
         return q
 
+    def query_finished_sessions(self, sessions: List[str] = None) -> dict:
+        q = self.query_exp
+        q["exp_finished"] = True
+        q["exp_aborted"] = False
+
+        if sessions is not None:
+            q["exp_session_id"] = {"$in": sessions}
+
+        return q
+
+    def find_finished_sessions(self, sessions: List[str] = None) -> Iterator[str]:
+        if self.method == "local":
+            return self._find_finished_sessions_local(sessions)
+        elif self.method == "mongo":
+            return self._find_finished_sessions_mongo(sessions)
+
     def find_active_sessions(self, sessions: List[str] = None) -> Iterator[str]:
         if self.method == "local":
             return self._find_active_sessions_local(sessions)
         elif self.method == "mongo":
             return self._find_active_sessions_mongo(sessions)
-            
+
     def _find_active_sessions_local(self, sessions: List[str] = None) -> Iterator[str]:
         for member in self.members():
             if member.status.active:
                 yield member.data.session_id
-    
+
+    def _find_finished_sessions_mongo(
+        self, sessions: List[str] = None
+    ) -> Iterator[str]:
+        q = self.query_finished_sessions(sessions)
+        cursor = self.exp.db_main.find(q, projection=["exp_session_id"])
+        for sessiondata in cursor:
+            status = SessionGroup([sessiondata["exp_session_id"]])
+            if status.finished(self.exp):
+                yield sessiondata["exp_session_id"]
+
     def _find_active_sessions_mongo(self, sessions: List[str] = None) -> Iterator[str]:
         q = self.query_active_sessions(sessions)
         cursor = self.exp.db_main.find(q, projection=["exp_session_id"])
         for sessiondata in cursor:
-            yield sessiondata["exp_session_id"]
+            status = SessionGroup([sessiondata["exp_session_id"]])
+            if status.pending(self.exp):
+                yield sessiondata["exp_session_id"]
 
-    def find_finished_sessions(self, sessions: List[str] = None) -> Iterator[str]:
-        if self.method == "local":
-            return self._find_finished_sessions_local(sessions)
-
-        elif self.method == "mongo":
-            return self._find_finished_sessions_mongo(sessions)
-    
     def _find_finished_sessions_local(self, sessions: List[str]) -> Iterator[str]:
         for m in self.members():
             if m.status.finished and m.data.session_id in sessions:
                 yield m.data.session_id
 
-    def _find_finished_sessions_mongo(self, sessions: List[str]) -> Iterator[str]:
-        q = self.query_exp
-        q["exp_finished"] = True
-        if sessions is not None:
-            q["exp_session_id"] = {"$in": sessions}
-        cursor = self.exp.db_main.find(q, projection=["exp_session_id"])
-        for sessiondata in cursor:
-            yield sessiondata["exp_session_id"]
-
     def active_sessions_projection(self, cache_length: int = 1) -> dict:
         now = time.time()
 
@@ -494,15 +503,15 @@ class MemberManager:
     def active(self) -> GroupMember:
         if self.method == "local":
             return self._active_local()
-        
+
         elif self.method == "mongo":
             return self._active_mongo()
-    
+
     def _active_local(self) -> Iterator[GroupMember]:
         for m in self.members():
             if m.status.active:
                 yield m
-    
+
     def _active_mongo(self) -> Iterator[GroupMember]:
         q = self.query_mm
         p = self.active_sessions_projection()
@@ -536,16 +545,16 @@ class MemberManager:
     def find(self, sessions: List[str]) -> Iterator[GroupMember]:
         if self.method == "local":
             return self._find_local(sessions)
-        
+
         elif self.method == "mongo":
-            return self._find_mongo(sessions)    
-    
+            return self._find_mongo(sessions)
+
     def _find_local(self, sessions: List[str]) -> Iterator[GroupMember]:
         data = self.mm.io.load().members.values()
         for mdata in data:
             if mdata["session_id"] in sessions:
                 yield GroupMember(matchmaker=self.mm, **mdata)
-    
+
     def _find_mongo(self, sessions: List[str]) -> Iterator[GroupMember]:
         q = self.query_mm
         p = {f"members.{sid}": True for sid in sessions}
