@@ -1,12 +1,18 @@
 from . import MatchMaker, NoMatch, ParallelSpec, SequentialSpec
 
-def get_group(exp, roles: list = None, ongoing_sessions_ok: bool = False, nslots: int = 5):
+
+def get_group(
+    exp, roles: list = None, ongoing_sessions_ok: bool = False, nslots: int = 5
+):
     roles = ["a", "b"] if roles is None else roles
-    spec = SequentialSpec(*roles, nslots=nslots, name="test", ongoing_sessions_ok=ongoing_sessions_ok)
+    spec = SequentialSpec(
+        *roles, nslots=nslots, name="test", ongoing_sessions_ok=ongoing_sessions_ok
+    )
     mm = MatchMaker(spec, exp=exp)
     group = mm.match_to("test")
     return group
-    
+
+
 def get_group_groupwise(exp_factory: callable, roles: list = None, nslots: int = 5):
     roles = roles if roles is not None else ["a", "b"]
     exp1 = exp_factory()
@@ -28,13 +34,14 @@ def get_group_groupwise(exp_factory: callable, roles: list = None, nslots: int =
 
     return group1
 
+
 def get_multiple_groups(*mm, specname: str = "test"):
 
     try:
         mm[0].match_to(specname)
     except NoMatch:
         pass
-    
+
     groups = [m.match_to(specname) for m in reversed(mm)]
 
-    return tuple(reversed(groups))
\ No newline at end of file
+    return tuple(reversed(groups))
